const express = require("express");
const passport = require("passport");
const router = express.Router();

const encrypt = require("../src/utils").encrypt;

const users = require("../models/user");

const check = require("../src/permissions").check_permissions;

router
  .get("/register", function(req, res, next) {
    res.render("register", {
      roles: Object.values(users.USER_ROLE),
      messages: req.flash("info")
    });
  })
  .post("/register", function(req, res, next) {
    users.User.findOne({ email: req.body.email })
      .then(u => {
        if (u) {
          req.flash("info", "Cette adresse email est déjà prise");
          res.redirect("/register");
        } else {
          const newUser = new users.User({
            name: req.body.name,
            role: req.body.role,
            email: req.body.email,
            password: encrypt(req.body.password)
          });
          return newUser.save();
        }
      })
      .then(
        u => {
          if (u) res.redirect("/login");
        },
        err => {
          Object.values(err.errors).forEach(err =>
            req.flash("info", err.message)
          );
          res.redirect("/register");
        }
      )
      .catch(next);
  })
  .get("/login", function(req, res, next) {
    res.render("login");
  })
  .post(
    "/login",
    passport.authenticate("local", {
      failureRedirect: "/login",
      successRedirect: "/"
    })
  )
  .get("/logout", function(req, res) {
    req.logout();
    res.redirect("/");
  });

module.exports = router;
